==> dynamichazard_0.5.0.tar.gz: dynamichazard/R/loglike.R <==
#' @title Log likelihood of mean path of \code{\link{ddhazard}} object
#'
#' @description
#' Computes the log likelihood of (a potentially new) data set given the estimated:
#' \deqn{E_{\theta}(\alpha_1 | y_{1:d}), E_{\theta}(\alpha_{2} | y_{1:d}), ..., E_{\theta}(\alpha_{d} | y_{1:d})}
#'
#' of the \code{ddhazard} object. Note that this is not the log likelihood of the observed data given the outcome.
#'
#' @param object an object of class \code{ddhazard}.
#' @param data new data to evaluate the likelihood for.
#' @param id the individual identifiers as in \code{\link{ddhazard}}.
#' @param ... unused.
#'
#' @examples
#'library(dynamichazard)
#'fit <- ddhazard(
#'  Surv(time, status == 2) ~ log(bili), pbc, id = pbc$id, max_T = 3600,
#'  Q_0 = diag(1, 2), Q = diag(1e-4, 2), by = 50,
#'  control = list(method = "GMA"))
#'logLik(fit)
#'
#' @export
logLik.ddhazard = function(object, data = NULL, id, ...){
  data <- if(!is.null(object$data)) object$data else data
  if(is.null(data))
    stop("data is needed to compute log likelihood. Please, pass the data set used in 'ddhazard' call")

  X <- get_design_matrix(
    data = data, Terms = object$terms, xlev = object$xlev,
    has_fixed_intercept = object$has_fixed_intercept)
  X$X <- t(X$X)

  fixed_effects_offsets <- if(ncol(X$fixed_terms) == 0)
    rep(0, nrow(X$fixed_terms)) else
      X$fixed_terms %*% object$fixed_effects

  risk_obj <- object$risk_set
  if(is.null(risk_obj)){
    if(missing(id))
      stop("id need to compute log likelihood. Please, pass the id used in 'ddhazard' call")

    if(object$model %in% exp_model_names){
      is_for_discrete_model <- F

    } else if (object$model == "logit"){
      is_for_discrete_model <- T

    } else
      stop("logLik not implemented for model '", object$model, "'")

    risk_obj <- get_risk_obj(
      Y = X$Y, by = unique(diff(object$times)), id = id,
      max_T = max(object$times), is_for_discrete_model = is_for_discrete_model)
  }

  val <- logLike_cpp(
    X = X$X, risk_obj = risk_obj, F = object$F_,
    Q_0 = object$Q_0, Q = object$Q, a_t_d_s = t(object$state_vecs),
    tstart = X$Y[, 1], tstop = X$Y[, 2], order_ = object$order,
    model = object$model, fixed_effects_offsets = fixed_effects_offsets)

  attr(val, "prior_loglike") <- val[2]
  val <- val[1]

  if(object$est_Q_0)
    warning("parameters for Q_0 are not included in attribute df")

  class(val) <- "logLik"
  val
}

#' @title Log likelihood of a \code{PF_clouds} object
#' @param object an object of class \code{PF_clouds}.
#' @param ... unused.
#' @description
#' Computes the log likelihood using the forward filter clouds. See the \code{vignette("Particle_filtering", "dynamichazard")} for details.
#'
#' @return
#' The log-likelihood value given the observed data and set of parameter used when simulating the clouds.
#'
#' @export
logLik.PF_clouds <- function(object, ...){
  sum(tail(
    sapply(lapply(object$forward_clouds,
                  "[[", "log_unnormalized_weights"),
           function(x){
             .max <- max(x)
             log(sum(exp(x - .max))) + .max - log(length(x))
           }),
    -1))
}

==> dynamichazard_0.5.2.tar.gz: dynamichazard/R/loglike.R <==
#' @title Log Likelihood of Mean Path of ddhazard Object
#'
#' @description
#' Computes the log likelihood of (a potentially new) data set given the estimated:
#' \deqn{E_{\theta}(\alpha_1 | y_{1:d}), E_{\theta}(\alpha_{2} | y_{1:d}), ..., E_{\theta}(\alpha_{d} | y_{1:d})}
#'
#' of the \code{ddhazard} object. Note that this is not the log likelihood of the observed data given the outcome.
#'
#' @param object an object of class \code{ddhazard}.
#' @param data new data to evaluate the likelihood for.
#' @param id the individual identifiers as in \code{\link{ddhazard}}.
#' @param ... unused.
#'
#' @examples
#'library(dynamichazard)
#'fit <- ddhazard(
#'  Surv(time, status == 2) ~ log(bili), pbc, id = pbc$id, max_T = 3600,
#'  Q_0 = diag(1, 2), Q = diag(1e-4, 2), by = 50,
#'  control = list(method = "GMA"))
#'logLik(fit)
#'
#' @export
logLik.ddhazard = function(object, data = NULL, id, ...){
  data <- if(!is.null(object$data)) object$data else data
  if(is.null(data))
    stop("data is needed to compute log likelihood. Please, pass the data set used in 'ddhazard' call")

  X <- get_design_matrix(
    data = data, Terms = object$terms, xlev = object$xlev,
    has_fixed_intercept = object$has_fixed_intercept)
  X$X <- t(X$X)

  fixed_effects_offsets <- if(ncol(X$fixed_terms) == 0)
    rep(0, nrow(X$fixed_terms)) else
      X$fixed_terms %*% object$fixed_effects

  risk_obj <- object$risk_set
  if(is.null(risk_obj)){
    if(missing(id))
      stop("id need to compute log likelihood. Please, pass the id used in 'ddhazard' call")

    if(object$model %in% exp_model_names){
      is_for_discrete_model <- F

    } else if (object$model == "logit"){
      is_for_discrete_model <- T

    } else
      stop("logLik not implemented for model '", object$model, "'")

    risk_obj <- get_risk_obj(
      Y = X$Y, by = unique(diff(object$times)), id = id,
      max_T = max(object$times), is_for_discrete_model = is_for_discrete_model)
  }

  val <- logLike_cpp(
    X = X$X, risk_obj = risk_obj, F = object$F_,
    Q_0 = object$Q_0, Q = object$Q, a_t_d_s = t(object$state_vecs),
    tstart = X$Y[, 1], tstop = X$Y[, 2], order_ = object$order,
    model = object$model, fixed_effects_offsets = fixed_effects_offsets)

  attr(val, "prior_loglike") <- val[2]
  val <- val[1]

  if(object$est_Q_0)
    warning("parameters for Q_0 are not included in attribute df")

  class(val) <- "logLik"
  val
}

#' @title Log Likelihood of a PF_clouds Object
#' @param object an object of class \code{PF_clouds}.
#' @param ... unused.
#' @description
#' Computes the log likelihood using the forward filter clouds. See the \code{vignette("Particle_filtering", "dynamichazard")} for details.
#'
#' @return
#' The log-likelihood value given the observed data and set of parameter used when simulating the clouds.
#'
#' @importFrom utils tail
#' @export
logLik.PF_clouds <- function(object, ...){
  sum(tail(
    sapply(lapply(object$forward_clouds,
                  "[[", "log_unnormalized_weights"),
           function(x){
             .max <- max(x)
             log(sum(exp(x - .max))) + .max - log(length(x))
           }),
    -1))
}
